%tangent magnetometer anaylysis
% by Tom Nottingham & Dr French. November 2017.

function tangent_magnetometer_analysis

%Set fontsize for graphs
fsize = 18;

%Number of evaluation points for predicted curve
N = 1000;

%Max B_M_by_B_E value
Bmax = 30;

%

%Read in Excel sheet containing measurements
[num,txt,raw] = xlsread('Tangent magnetometer.xlsx');

%Distance /m between N pole of magnet and the centre of the magnetometer.
%Sort in ascending order.
x = num(:,2);
[x,i] = sort(x);

%Angle of deflection /deg of needle from original magnetic north position
theta = num(:,3);
theta = theta(i);

%Determine magnetic field strength of bar magnet B_M at centre of magnetometer,
%in terms of the strength of the Earth's magnetic field strength B_E
B_M_by_B_E = tan(theta*pi/180);

%Determine power law of B_M variation with distance using log,log graph.
Y = log( B_M_by_B_E(2:end) );
X = log( x(2:end) );

%Compute line of best fit of Y vs X. 
[Yfit,Xfit, r,Ybar,Xbar,M,C] = bestfit(X,Y);

%Hence determine predicted variation of B_M_by_B_E vs x
% B_M_by_B_E = k*x^M
k = exp(C);
xx = linspace(0,max(x),N);
B_M_by_B_E_prediction =  k*(xx.^M);

%Compute line of best fit of tan(theta) vs 1./x^3
XX = x(2:end).^(-3);
YY = B_M_by_B_E(2:end);
[YYfit,XXfit,rr,YYbar,XXbar,kk,CC] = bestfit(XX,YY);

%Hence determine predicted variation of B_M_by_B_E vs x
% B_M_by_B_E = k*x^-3
xx = linspace(0,max(x),N);
B_M_by_B_E_prediction2 =  kk*(xx.^-3);

%

%Plot deflection angle vs distance x and save PNG file
fig = figure('color',[1 1 1],'name','tangent magnetometer');
plot(x,theta,'b+')
grid on
box on
xlabel('Distance of bar magnet from compass /m','fontsize',fsize)
ylabel('Magnetometer deflection /^o','fontsize',fsize)
title('Tangent magnetometer deflection using bar magnet','fontsize',fsize)
set(gca,'fontsize',fsize )
grid on
box on
print(fig,'Tangent Magnetometer angle vs x.png','-dpng','-r500')
clf;

%

%Plot log-log graph and determine line of best fit to work out power law
plot(Xfit, Yfit, 'k-', X,Y,'b+' )
grid on
box on
xlabel('X = log( magnet distance x/m )','fontsize',fsize)
ylabel('Y = log( tan\theta )','fontsize',fsize)
title({'log vs log graph to determine magnetic field power law',...
    ['Y = ',num2str(M,3),'X ',sign_str(C),num2str(abs(C),3)] },'fontsize',fsize)
set(gca,'fontsize',fsize )
grid on
box on
print(fig,'Tangent Magnetometer log vs log.png','-dpng','-r500')
clf;

%Plot B_M_by_B_E vs distance x and overlay power-law prediction
plot(xx,B_M_by_B_E_prediction,'r-',x,B_M_by_B_E,'b+')
ylim([0,Bmax])
grid on
box on
xlabel('Distance x of bar magnet from compass /m','fontsize',fsize)
ylabel('B_M/B_E','fontsize',fsize)
title(['B_M/B_E = tan\theta = ',num2str(k,2),'/x^{',num2str(-M,2),'}'],...
    'fontsize',fsize)
set(gca,'fontsize',fsize )
grid on
box on
print(fig,'Tangent Magnetometer B vs x.png','-dpng','-r500')
clf;

%Plot tan(theta) vs 1/x^3 to work out constant kk
plot(XXfit,YYfit,'k-',XX,YY,'b+')
ylim([0,Bmax])
grid on
box on
xlabel('1/x^3','fontsize',fsize)
ylabel('B_M/B_E','fontsize',fsize)
title(['Y = ',num2str(kk,3),'X ',sign_str(C),num2str(abs(CC),3)],...
    'fontsize',fsize)
set(gca,'fontsize',fsize )
grid on
box on
print(fig,'Tangent Magnetometer B vs x^-3.png','-dpng','-r500')
clf;

%Plot B_M_by_B_E vs distance x and overlay power-law prediction
plot(xx,B_M_by_B_E_prediction2,'r-',x,B_M_by_B_E,'b+')
ylim([0,Bmax])
grid on
box on
xlabel('Distance x of bar magnet from compass /m','fontsize',fsize)
ylabel('B_M/B_E','fontsize',fsize)
title(['B_M/B_E = tan\theta = ',num2str(kk,2),'/x^3'],...
    'fontsize',fsize)
set(gca,'fontsize',fsize )
grid on
box on
print(fig,'Tangent Magnetometer B vs x v2.png','-dpng','-r500')
close(fig);

%%

%Line of best fit function yfit = m*x +c, with product moment correlation
%coefficient r
function [yfit,xfit,r,ybar,xbar,m,c] = bestfit(x,y)
%Find any x or y values that are NaN or Inf
i = find( isnan(abs(x)) | isnan(abs(y)) | ( abs(x)==Inf ) | ( abs(y)==Inf ) ) ;
x(i) = [];
y(i) = [];
n = length(x);
xbar = mean(x);
ybar = mean(y);
xybar = mean(x.*y);
Vx = (1/n)*sum(x.^2 ) - xbar^2;
Vy = (1/n)*sum(y.^2 ) - ybar^2;
COVxy = (1/n)*sum(x.*y) - xbar*ybar;
m = COVxy/Vx;
c = ybar - m*xbar;
r = COVxy/sqrt( Vx*Vy );
yfit = m*x + c;
xfit = x;

%

%Sign string of number
function s = sign_str(x)
if x<0
    s = ' - ';
else
    s = ' + ';
end
%End of code
